#
# This file is licensed under the 3-clause BSD license.
# Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.
# See LICENSE.txt for details.
#
macro(import_gui)
  # If the target already exists, do nothing
  if(NOT TARGET Scine::GUI OR NOT TARGET Scine::Kernel)
    # Try to find the package locally
    find_package(Scine OPTIONAL_COMPONENTS GUI QUIET)
    find_package(Scine OPTIONAL_COMPONENTS Kernel QUIET)
    if(TARGET Scine::GUI AND TARGET Scine::Kernel)
      message(STATUS "Scine::GUI and Scine::Kernel found locally at ${Scine_DIR}")
    else()
      # Download it instead
      include(DownloadProject)
      download_project(
        PROJ scine-gui-kernel
        GIT_REPOSITORY git@gitlab.chab.ethz.ch:scine/gui-kernel.git
        GIT_TAG        develop
        QUIET
      )
      # Note: Options defined in the project calling this function override default
      # option values specified in the imported project.
      add_subdirectory(${scine-gui-kernel_SOURCE_DIR} ${scine-gui-kernel_BINARY_DIR})

      # Final check if all went well
      if(TARGET Scine::GUI AND TARGET Scine::Kernel)
        message(STATUS
          "Scine::GUI and Scine::Kernel were not found in your PATH, so it was downloaded."
        )
      else()
        string(CONCAT error_msg
          "Scine::GUI and Scine::Kernel were not found in your PATH and could not be"
          "established through a download. Try specifying Scine_DIR or altering "
          "CMAKE_PREFIX_PATH to point to a candidate Scine installation base "
          "directory."
        )
        message(FATAL_ERROR ${error_msg})
      endif()
    endif()
  endif()
endmacro()
