/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */
#ifndef INTEGRALS_INTEGRALSDENSITYMATRIXGUESSGENERATOR_H
#define INTEGRALS_INTEGRALSDENSITYMATRIXGUESSGENERATOR_H

#include <Utils/Scf/MethodInterfaces/DensityMatrixGuessCalculator.h>

namespace Scine {
namespace Utils {
class OverlapCalculator;
} // namespace Utils
namespace Sparrow {
namespace Integrals {
class BasisSetController;

enum class InitialGuessType { ExtendedHueckel, HCore, SemiEmpirical };

class AbInitioDensityMatrixGuessGenerator : public Utils::DensityMatrixGuessCalculator {
 public:
  explicit AbInitioDensityMatrixGuessGenerator(const BasisSetController& basisSet, const int& nElectrons,
                                               Utils::OverlapCalculator& overlapCalculator);

  ~AbInitioDensityMatrixGuessGenerator() final;

  void setGuessType(InitialGuessType guessType);
  Utils::DensityMatrix calculateGuess() const final;

 private:
  Utils::DensityMatrix calculateExtendedHueckelGuess() const;
  Utils::DensityMatrix calculateHCoreGuess() const;
  //  Utils::DensityMatrix calculateSemiempiricalGuess() const;
  InitialGuessType initialGuessType_;
  const int& nElectrons_;
  const BasisSetController& basisSet_;
  Utils::OverlapCalculator& overlapCalculator_;
};

} // namespace Integrals
} // namespace Sparrow
} // namespace Scine

#endif // INTEGRALS_INTEGRALSDENSITYMATRIXGUESSGENERATOR_H
