/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */
#ifndef INTEGRALS_ERISAVERDIGESTER_H
#define INTEGRALS_ERISAVERDIGESTER_H

#include "../BasisSetController.h"
#include "Orbitals.h"
#include <memory>

namespace Scine {
namespace Utils {
class AtomsOrbitalsIndexes;
}
namespace Sparrow {
namespace Integrals {

class BasisSetController;

class NddoEriSaverDigester {
 public:
  using IndexType = std::array<int, 4>;
  explicit NddoEriSaverDigester(const BasisSetController& basisSet);
  ~NddoEriSaverDigester();
  NddoEriSaverDigester(NddoEriSaverDigester&& rhs) noexcept;
  NddoEriSaverDigester& operator=(NddoEriSaverDigester&& rhs) = delete;
  NddoEriSaverDigester(const NddoEriSaverDigester& rhs) = delete;
  NddoEriSaverDigester& operator=(const NddoEriSaverDigester& rhs) = delete;

  void digest(const Eigen::RowVectorXd& bufferVector, IndexType shellIndices, int atom_i, int atom_j);
  void digest(const Eigen::MatrixXd& oneCenterBlock, int atom_i);

  const EriType& getResult() const;
  void initialize(int numberThreads);

 private:
  EriType eri_;
  const BasisSetController& basisSet_;
  const std::vector<size_t>& indexFirstBFInShell_;
  std::unique_ptr<Utils::AtomsOrbitalsIndexes> aoInfos_;
};

} // namespace Integrals
} // namespace Sparrow
} // namespace Scine

#endif // INTEGRALS_ERISAVERDIGESTER_H
