/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */

#ifndef SPARROW_MNDOMETHOD_H
#define SPARROW_MNDOMETHOD_H

#include <Sparrow/Implementations/Nddo/ObjectWithIntegrals.h>
#include <Utils/Scf/MethodInterfaces/ScfMethod.h>

namespace Scine {
namespace Utils {
enum class DerivativeOrder;
}
namespace Sparrow {

namespace nddo {
class FockMatrix;
class NDDOInitializer;
struct Parameters;
class OneElectronMatrix;
class TwoElectronMatrix;

class MNDOMethod : public Utils::ScfMethod, public ObjectWithIntegrals {
 public:
  MNDOMethod();
  ~MNDOMethod() override;

  /*! Deprecated! Initialize the method from a structure and a parameter file. */
  void setStructure(const Utils::AtomCollection& atoms, std::string parameterPath = "");
  /*! Load the parameters from a file. */
  void readParameters(const std::string& parameterPath);
  /*! Save the parameters to a file. */
  void saveParameters(const std::string& fileName);

  NDDOInitializer& getInitializer() {
    return *mndoSettings_;
  }
  const NDDOInitializer& getInitializer() const {
    return *mndoSettings_;
  }

  /*! Get reference to the class for raw MNDO parameters. */
  Parameters& getRawParameters();
  /*! Get const reference to the class for raw MNDO parameters. */
  const Parameters& getRawParameters() const;

  const nddo::OneElectronMatrix& getOneElectronMatrix() const final;
  const nddo::TwoElectronMatrix& getTwoElectronMatrix() const final;

  auto evaluatePotentialForDensity(const Utils::DensityMatrix& density) const -> Utils::SpinAdaptedMatrix;

 private:
  std::shared_ptr<NDDOInitializer> mndoSettings_;
  std::shared_ptr<FockMatrix> mndoFock_;
};

} // namespace nddo

} // namespace Sparrow
} // namespace Scine
#endif // SPARROW_MNDOMETHOD_H
