/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */

#ifndef SPARROW_OBJECTWITHINTEGRALS_H
#define SPARROW_OBJECTWITHINTEGRALS_H

#include <Integrals/TwoBodiesIntegrals/Orbitals.h>
#include <Utils/DataStructures/AtomsOrbitalsIndexes.h>
#include <Utils/Typenames.h>

namespace Scine {
namespace Sparrow {
namespace nddo {
class OneElectronMatrix;
class TwoElectronMatrix;
class OneCenterIntegralContainer;
class TwoCenterIntegralContainer;

class ObjectWithIntegrals {
 public:
  virtual ~ObjectWithIntegrals() = default;
  virtual auto getOneElectronMatrix() const -> const nddo::OneElectronMatrix& = 0;
  virtual auto getTwoElectronMatrix() const -> const nddo::TwoElectronMatrix& = 0;
  /**
   * @brief Transforms integrals to the EriType data structure.
   */
  auto transformIntegrals(const Utils::ElementTypeCollection& elements, const Utils::AtomsOrbitalsIndexes& aoInfos) const
      -> Integrals::EriType;
};

} // namespace nddo
} // namespace Sparrow
} // namespace Scine

#endif // SPARROW_OBJECTWITHINTEGRALS_H
