/**
 * @file AbInitioOverlapCalculator.h
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher
 * Group.\n See LICENSE.txt for details.
 */

#include "AbInitioOverlapCalculator.h"
#include <Integrals/BasisSetController.h>
#include <Integrals/Libint.h>
#include <Utils/Math/DerivOrderEnum.h>

namespace Scine {
namespace Sparrow {
namespace Integrals {

AbInitioOverlapCalculator::AbInitioOverlapCalculator(const BasisSetController& basisSet) : basisSet_(basisSet) {
}

void AbInitioOverlapCalculator::calculateOverlap(Utils::DerivativeOrder highestRequiredOrder) {
  auto& libint = Integrals::Libint::getInstance();
  overlap_.reset(basisSet_.nBasisFunction());
  overlap_.setOrder(highestRequiredOrder);

  if (highestRequiredOrder == Utils::DerivativeOrder::Zero) {
    overlap_ = libint.compute1BodyIntegrals<Operator::overlap, Utils::DerivativeOrder::Zero>(basisSet_);
  }
  else if (highestRequiredOrder == Utils::DerivativeOrder::One) {
    overlap_ = libint.compute1BodyIntegrals<Operator::overlap, Utils::DerivativeOrder::One>(basisSet_);
  }
  else {
    overlap_ = libint.compute1BodyIntegrals<Operator::overlap, Utils::DerivativeOrder::Two>(basisSet_);
  }
}

const Utils::MatrixWithDerivatives& AbInitioOverlapCalculator::getOverlap() const {
  return overlap_;
}

void AbInitioOverlapCalculator::reset() {
  overlap_.reset(basisSet_.nBasisFunction());
  overlap_.setOrder(Utils::DerivativeOrder::Zero);
}

} // namespace Integrals
} // namespace Sparrow
} // namespace Scine
