/**
 * @file Calculator.h
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher
 * Group.\n See LICENSE.txt for details.
 */

#ifndef SPARROW_ABINITIOOVERLAPCALCULATOR_H
#define SPARROW_ABINITIOOVERLAPCALCULATOR_H
#include <Utils/DataStructures/MatrixWithDerivatives.h>
#include <Utils/Scf/MethodInterfaces/OverlapCalculator.h>

namespace Scine {
namespace Utils {
enum class derivOrder;
} // namespace Utils
namespace Sparrow {
namespace Integrals {

class BasisSetController;

class AbInitioOverlapCalculator final : public Utils::OverlapCalculator {
 public:
  explicit AbInitioOverlapCalculator(const BasisSetController& basisSet);
  ~AbInitioOverlapCalculator() final = default;
  /**
   * @brief Calculate the overlap up to the highest required order.
   */
  void calculateOverlap(Utils::DerivativeOrder highestRequiredOrder) final;
  /**
   * @brief Getter for the overlap matrix up to the requested order.
   */
  const Utils::MatrixWithDerivatives& getOverlap() const final;
  /**
   * @brief Method to reinitialize the overlap matrix to an empty one with no derivatives.
   */
  void reset() final;

 private:
  const BasisSetController& basisSet_;
  Utils::MatrixWithDerivatives overlap_;
};

} // namespace Integrals
} // namespace Sparrow
} // namespace Scine
#endif // SPARROW_ABINITIOOVERLAPCALCULATOR_H
