/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */

#include "CoreElectronEvaluation.h"
#include "KlopmanFinalizer.h"
#include "OmxElementParameters.h"
#include <Integrals/BasisSetController.h>
#include <Integrals/Libint.h>
#include <Utils/Geometry/ElementInfo.h>
namespace Scine {
namespace Sparrow {
namespace nddo {
namespace CoreElectronEvaluation {

auto analyticalCoreElectron(int atom_a, int atom_b, double chargeOnB, const Integrals::BasisSetController& basisSet,
                            const KlopmanFinalizer& kofactors) -> Eigen::MatrixXd {
  Eigen::MatrixXd result =
      Integrals::Libint::getInstance().computeVuvB<Utils::DerivativeOrder::Zero>(atom_a, atom_b, chargeOnB, basisSet)[0];
  Integrals::DiatomicKey index{atom_a, atom_b};
  Integrals::canonicalize(index);
  if (kofactors.has<Utils::DerivativeOrder::Zero>(index)) {
    result *= kofactors.get<Utils::DerivativeOrder::Zero>(index);
  }
  return result;
}

auto getSemiempiricalCoreElectron(int mu, int nu, Integrals::DiatomicKey index, const Integrals::EriType& eri,
                                  double coreCharge) -> double {
  bool transpose = index.second > index.first;
  Integrals::canonicalize(index);
  using namespace Integrals::Orbitals;

  assert((transpose && eri.at(index).get0().rows() > OrbitalPair::s_s &&
          eri.at(index).get0().cols() > pair(Orbital(mu), Orbital(nu))) ||
         (!transpose && eri.at(index).get0().cols() > OrbitalPair::s_s &&
          eri.at(index).get0().rows() > pair(Orbital(mu), Orbital(nu))));

  const double integral = transpose ? eri.at(index).get0()(OrbitalPair::s_s, pair(Orbital(mu), Orbital(nu)))
                                    : eri.at(index).get0()(pair(Orbital(mu), Orbital(nu)), OrbitalPair::s_s);
  return -integral * coreCharge;
}

} // namespace CoreElectronEvaluation
} // namespace nddo
} // namespace Sparrow
} // namespace Scine
