/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */

#include "KlopmanFinalizer.h"
#include <Utils/Math/DerivOrderEnum.h>
#include <Utils/Typenames.h>
#include <boost/optional.hpp>
namespace Scine {
namespace Utils {
class MatrixWithDerivatives;
} // namespace Utils
namespace Sparrow {
namespace Integrals {
class Libint;
class BasisSetController;
} // namespace Integrals

namespace nddo {

class OmxElementParameters;

namespace CoreElectronEvaluation {

auto analyticalCoreElectron(int atom_a, int atom_b, double chargeOnB, const Integrals::BasisSetController& basisSet,
                            const KlopmanFinalizer& koFactor = {}) -> Eigen::MatrixXd;
/**
 * @brief get first derivative of the analytical core-electron
 */
inline auto analyticalCoreElectronFirst(int /*atom_a*/, int /*atom_b*/, double /*chargeOnB*/,
                                        const Integrals::BasisSetController& /*basisSet*/, const KlopmanFinalizer &
                                        /*kofactors = {}*/) -> Eigen::MatrixXd {
  throw std::runtime_error("Derivatives not implemented");
}

/// TODO: Derivatives: needs derivatives of kofactors!
/**
 * Calculates Z_B * <muI nuI | sJ sJ >. 2elIntegral according to OMx integrals.
 * index: second index is the one that provides the core, first one refers to mu and nu.
 * Internally reordered for eri (eri are symmetrized!).
 */
auto getSemiempiricalCoreElectron(int mu, int nu, Integrals::DiatomicKey index, const Integrals::EriType& eri,
                                  double coreCharge) -> double;
/**
 * @brief gets first derivative of the semiempirical core-electron
 */
inline auto getSemiempiricalCoreElectronFirst(int /*mu*/, int /*nu*/, Integrals::DiatomicKey /*index*/,
                                              const Integrals::EriType& /*eri*/, double /*coreCharge*/) -> double {
  throw std::runtime_error("Derivatives not implemented");
}

} // namespace CoreElectronEvaluation
} // namespace nddo
} // namespace Sparrow
} // namespace Scine
