#include "Dftd4Correction.h"

#include <fstream>
#include <boost/dll.hpp>
#include <boost/filesystem.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/uuid/uuid.hpp>
#include <boost/uuid/uuid_generators.hpp>
#include <boost/uuid/uuid_io.hpp>

#include <Utils/IO/ChemicalFileFormats/XyzStreamHandler.h>
#include <Utils/IO/NativeFilenames.h>
#include <iostream>

namespace Scine {
namespace Sparrow {
namespace nddo {

void Dftd4Correction::calculate() {
  setUpCalculation();

  std::string programLocation = boost::dll::program_location().parent_path().string();
  std::string executable = programLocation + "/dftd4";
  externalProgram_.executeCommand(executable + " --charge " + std::to_string(charge_) + " -f wb97x --silent " + workingDirectory_ + "molecule.xyz");

  parseResult();
  
  cleanUpCalculation();
}

double Dftd4Correction::getDispersionEnergy() {
  return dispersionEnergy_;
}

// Backported from develop branch of Utils
// TODO: Remove once no longer necessary
std::string Dftd4Correction::createRandomDirectoryName(const std::string& baseDirectory) {
  boost::uuids::uuid uuid = boost::uuids::random_generator()();
  std::string uuidString = boost::uuids::to_string(uuid);
  auto directoryName = Utils::NativeFilenames::combinePathSegments(baseDirectory, uuidString);
  return Utils::NativeFilenames::addTrailingSeparator(directoryName);
}

void Dftd4Correction::setUpCalculation() {
  workingDirectory_ = createRandomDirectoryName("/tmp");
  externalProgram_.setWorkingDirectory(workingDirectory_);
  externalProgram_.createWorkingDirectory();
  std::ofstream output(workingDirectory_ + "molecule.xyz");
  Utils::XyzStreamHandler::write(output, *atomCollection_);
}

void Dftd4Correction::cleanUpCalculation() {
  boost::filesystem::remove_all(workingDirectory_);
}

void Dftd4Correction::parseResult() {
  std::ifstream input(workingDirectory_ + ".EDISP");
  input >> dispersionEnergy_;
  input.close();
}

}
}
}
