/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */

#ifndef SPARROW_OMXDFTD4CORRECTION_H
#define SPARROW_OMXDFTD4CORRECTION_H

#include <memory>
#include <string>
#include <Utils/Geometry/AtomCollection.h>
#include <Utils/ExternalQC/ExternalProgram.h>

namespace Scine {
namespace Sparrow {
namespace nddo {

/**
 * @class Dftd4Correction
 * @brief Class invoking the dftd4 binary to calculate D4 dispersion corrections
 */
class Dftd4Correction {
  public:
    Dftd4Correction() = delete;

    Dftd4Correction(const std::shared_ptr<Utils::AtomCollection> atomCollection, int charge) {
      atomCollection_ = std::move(atomCollection);
      charge_ = charge;
    }

    ~Dftd4Correction() = default;

    void calculate();

    double getDispersionEnergy();

  private:
    std::shared_ptr<Utils::AtomCollection> atomCollection_;
    int charge_ = 0;
    double dispersionEnergy_ = 0.0;
    Utils::ExternalQC::ExternalProgram externalProgram_ = Utils::ExternalQC::ExternalProgram();
    std::string workingDirectory_;

    // Backported from develop branch of Utils
    // TODO: Remove once no longer necessary
    std::string createRandomDirectoryName(const std::string& baseDirectory);

    void setUpCalculation();

    void cleanUpCalculation();

    void parseResult();
};

}
}
}

#endif // SPARROW_OMXDFTD4CORRECTION_H
