/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */

#ifndef SPARROW_KLOPMANOHNOOBSERVER_H
#define SPARROW_KLOPMANOHNOOBSERVER_H

#include "KlopmanFinalizer.h"
#include <memory>

namespace Scine {
namespace Sparrow {
namespace nddo {
/**
 * @brief Struct for the observation of changes in the KlopmanFinalizer.
 * This is needed because I did not want to make the Fock Matrix dependent on the
 * repulsion or vice versa, and the calling code is in utils, so this serves a purpose.
 */
struct KlopmanOhnoObserver {
  virtual ~KlopmanOhnoObserver() = default;
  virtual void update(std::shared_ptr<KlopmanFinalizer> koFactors) = 0;
};

/**
 * @brief Struct for the observation of changes in the KlopmanFinalizer.
 * This is needed because I did not want to make the Fock Matrix dependent on the
 * repulsion or vice versa, and the calling code is in utils, so this serves a purpose.
 * This does not support the fancy unsubscribe(ptrType) function, as I just do not
 * need it.
 */
struct KlopmanOhnoShower {
  virtual ~KlopmanOhnoShower() = default;

  /**
   * @brief Add a subscriber.
   */
  void subscribe(std::shared_ptr<KlopmanOhnoObserver> observer) {
    observers_.push_back(std::move(observer));
  }

  /**
   * @brief Notify the subscribers that there is a new koFactor.
   */
  void notify(const std::shared_ptr<KlopmanFinalizer>& koFactors) {
    for (const auto& observer : observers_) {
      observer->update(koFactors);
    }
  }

 protected:
  std::vector<std::shared_ptr<KlopmanOhnoObserver>> observers_;
};
} // namespace nddo
} // namespace Sparrow
} // namespace Scine
#endif // SPARROW_KLOPMANOHNOOBSERVER_H
