#include "NNCorrection.h"

#include <fstream>
#include <boost/dll.hpp>
#include <boost/filesystem.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/uuid/uuid.hpp>
#include <boost/uuid/uuid_generators.hpp>
#include <boost/uuid/uuid_io.hpp>

#include <Utils/IO/ChemicalFileFormats/XyzStreamHandler.h>
#include <Utils/IO/NativeFilenames.h>
#include <iostream>

namespace Scine {
namespace Sparrow {
namespace nddo {

void NNCorrection::calculate() {
  setUpCalculation();

  std::string programLocation = boost::dll::program_location().parent_path().string();
  std::string executable = programLocation + "/AIQM1NNpart";
  std::string modelPath = programLocation + "/AIQM1.pt";
  externalProgram_.executeCommand(executable +  " " + modelPath + " molecule.xyz");

  parseResult();
  
  cleanUpCalculation();
}

double NNCorrection::getNNenergy() {
  return nnEnergy_;
}

std::string NNCorrection::createRandomDirectoryName(const std::string& baseDirectory) {
  boost::uuids::uuid uuid = boost::uuids::random_generator()();
  std::string uuidString = boost::uuids::to_string(uuid);
  auto directoryName = Utils::NativeFilenames::combinePathSegments(baseDirectory, uuidString);
  return Utils::NativeFilenames::addTrailingSeparator(directoryName);
}

void NNCorrection::setUpCalculation() {
  workingDirectory_ = createRandomDirectoryName("/tmp");
  externalProgram_.setWorkingDirectory(workingDirectory_);
  externalProgram_.createWorkingDirectory();
  std::ofstream output(workingDirectory_ + "molecule.xyz");
  Utils::XyzStreamHandler::write(output, *atomCollection_);
}

void NNCorrection::cleanUpCalculation() {
  boost::filesystem::remove_all(workingDirectory_);
}

void NNCorrection::parseResult() {
  std::ifstream input(workingDirectory_ + ".Enn");
  input >> nnEnergy_;
  input.close();
}

}
}
}
