/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */

#ifndef SPARROW_NNCORRECTION_H
#define SPARROW_NNCORRECTION_H

#include <memory>
#include <string>
#include <Utils/Geometry/AtomCollection.h>
#include <Utils/ExternalQC/ExternalProgram.h>

namespace Scine {
namespace Sparrow {
namespace nddo {

/**
 * @class NNCorrection
 * @brief Class invoking the AIQM1NNpart binary to calculate neural network corrections
 */
class NNCorrection {
  public:
    NNCorrection() = delete;

    NNCorrection(const std::shared_ptr<Utils::AtomCollection> atomCollection) {
      atomCollection_ = std::move(atomCollection);
    }

    ~NNCorrection() = default;

    void calculate();

    double getNNenergy();

  private:
    std::shared_ptr<Utils::AtomCollection> atomCollection_;
    double nnEnergy_ = 0.0;
    Utils::ExternalQC::ExternalProgram externalProgram_ = Utils::ExternalQC::ExternalProgram();
    std::string workingDirectory_;

    std::string createRandomDirectoryName(const std::string& baseDirectory);

    void setUpCalculation();

    void cleanUpCalculation();

    void parseResult();
};

}
}
}

#endif // SPARROW_NNCORRECTION_H
