/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */

#include "Sparrow/Implementations/Nddo/Omx/ParametersOmx.h"
#include "boost/filesystem.hpp"
#include <cereal/archives/json.hpp>
#include <cereal/cereal.hpp>
#include <cereal/types/unordered_map.hpp>
#include <cereal/types/utility.hpp>
#include <fstream>

namespace Scine {
namespace Sparrow {
namespace nddo {

/* Implementation of all archive templates */
template<typename Archive>
void ParametersOmx::Values::Sp::serialize(Archive& archive) {
  archive(CEREAL_NVP(s), CEREAL_NVP(p));
}

template void ParametersOmx::Values::Sp::serialize<cereal::JSONOutputArchive>(cereal::JSONOutputArchive&);
template void ParametersOmx::Values::Sp::serialize<cereal::JSONInputArchive>(cereal::JSONInputArchive&);

template<typename Archive>
void ParametersOmx::Values::Sppi::serialize(Archive& archive) {
  archive(CEREAL_NVP(s), CEREAL_NVP(pSigma), CEREAL_NVP(pPi));
}

template void ParametersOmx::Values::Sppi::serialize<cereal::JSONOutputArchive>(cereal::JSONOutputArchive&);
template void ParametersOmx::Values::Sppi::serialize<cereal::JSONInputArchive>(cereal::JSONInputArchive&);

template<typename Archive>
void ParametersOmx::Values::ResonanceIntegrals::serialize(Archive& archive) {
  archive(CEREAL_NVP(alpha), CEREAL_NVP(beta), CEREAL_NVP(alphaXH), CEREAL_NVP(betaXH));
}

template void ParametersOmx::Values::ResonanceIntegrals::serialize<cereal::JSONOutputArchive>(cereal::JSONOutputArchive&);
template void ParametersOmx::Values::ResonanceIntegrals::serialize<cereal::JSONInputArchive>(cereal::JSONInputArchive&);

template<typename Archive>
void ParametersOmx::Values::OrthogonalizationFactors::serialize(Archive& archive) {
  archive(CEREAL_NVP(f1), CEREAL_NVP(g1), CEREAL_NVP(f2), CEREAL_NVP(g2));
}

template void ParametersOmx::Values::OrthogonalizationFactors::serialize<cereal::JSONOutputArchive>(cereal::JSONOutputArchive&);
template void ParametersOmx::Values::OrthogonalizationFactors::serialize<cereal::JSONInputArchive>(cereal::JSONInputArchive&);

template<typename Archive>
void ParametersOmx::Values::Pseudopotential::serialize(Archive& archive) {
  archive(CEREAL_NVP(exponent), CEREAL_NVP(faa), CEREAL_NVP(alpha), CEREAL_NVP(beta));
}

template void ParametersOmx::Values::Pseudopotential::serialize<cereal::JSONOutputArchive>(cereal::JSONOutputArchive&);
template void ParametersOmx::Values::Pseudopotential::serialize<cereal::JSONInputArchive>(cereal::JSONInputArchive&);

template<typename Archive>
void ParametersOmx::Values::OneCenterTwoElectronIntegrals::serialize(Archive& archive) {
  archive(CEREAL_NVP(gss), CEREAL_NVP(gpp), CEREAL_NVP(gsp), CEREAL_NVP(gp2), CEREAL_NVP(hsp));
}

template void
ParametersOmx::Values::OneCenterTwoElectronIntegrals::serialize<cereal::JSONOutputArchive>(cereal::JSONOutputArchive&);
template void
ParametersOmx::Values::OneCenterTwoElectronIntegrals::serialize<cereal::JSONInputArchive>(cereal::JSONInputArchive&);

template<typename Archive>
void ParametersOmx::Values::serialize(Archive& archive) {
  archive(CEREAL_NVP(resonance), CEREAL_NVP(orthogonalization), CEREAL_NVP(pseudopotential),
          CEREAL_NVP(oneCenterTwoElectrons), CEREAL_NVP(oneCenterEnergy), CEREAL_NVP(exponent));
}

template void ParametersOmx::Values::serialize<cereal::JSONOutputArchive>(cereal::JSONOutputArchive&);
template void ParametersOmx::Values::serialize<cereal::JSONInputArchive>(cereal::JSONInputArchive&);

ParametersOmx ParametersOmx::read(const std::string& filename) {
  if (!boost::filesystem::exists(filename)) {
    throw std::runtime_error("Parameter file (" + filename + ") to read does not exist");
  }

  std::ifstream infile(filename);
  cereal::JSONInputArchive archive(infile);
  ParametersOmx result;
  archive(cereal::make_nvp("atomic", result.atomic));
  return result;
}

void ParametersOmx::write(const std::string& filename) const {
  std::ofstream outfile(filename);
  cereal::JSONOutputArchive archive(outfile);
  archive(cereal::make_nvp("atomic", atomic));
}

} // namespace nddo
} // namespace Sparrow
} // namespace Scine
