/**
 * @file
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */
#ifndef SPARROW_STANDARDTWOELECTRONMATRIXEVALUATOR_H
#define SPARROW_STANDARDTWOELECTRONMATRIXEVALUATOR_H

#include "TwoElectronMatrixConstructor.h"
#include <Utils/DataStructures/AtomsOrbitalsIndexes.h>
#include <Utils/DataStructures/MatrixWithDerivatives.h>
#include <Utils/DataStructures/SpinAdaptedMatrix.h>
#include <boost/functional/hash.hpp>
#include <unordered_map>

namespace Scine {

namespace Utils {
class DensityMatrix;
} // namespace Utils

namespace Sparrow {

namespace Integrals {
using DiatomicKey = std::pair<int, int>;
using EriType = std::unordered_map<DiatomicKey, Utils::MatrixWithDerivatives, boost::hash<DiatomicKey>>;
} // namespace Integrals

class TwoElectronMatrixEvaluator {
 public:
  TwoElectronMatrixEvaluator(const Integrals::EriType& eris, const Utils::DensityMatrix& densityMatrix,
                             Utils::AtomsOrbitalsIndexes aoInfos);

  void initialize(Utils::SpinAdaptedMatrix& matrix) const;
  Utils::SpinAdaptedMatrix evaluate() const;
  Utils::SpinAdaptedMatrix contract() const;
  Utils::SpinAdaptedMatrix evaluateDebug() const;

  // AddDerivatives? Call TwoElectronMatrixConstructor addDerivatives...

 private:
  const Integrals::EriType& eris_;
  const Utils::DensityMatrix& densityMatrix_;
  Utils::AtomsOrbitalsIndexes aoInfos_;
};

} // namespace Sparrow
} // namespace Scine

#endif // SPARROW_STANDARDTWOELECTRONMATRIXEVALUATOR_H
