/**
 * @file UnoCAS.h
 * @copyright This code is licensed under the 3-clause BSD license.\n
 *            Copyright ETH Zurich, Laboratory of Physical Chemistry, Reiher Group.\n
 *            See LICENSE.txt for details.
 */
#ifndef SPARROW_UNOCAS_H
#define SPARROW_UNOCAS_H

#include <Core/Log.h>
#include <Sparrow/Implementations/OrbitalSteeringSettings.h>
#include <Utils/DataStructures/SecondQuantization/CasSpecifier.h>
#include <Utils/DataStructures/SecondQuantization/UnoCI.h>
#include <memory>

namespace Scine {
namespace Core {
class Calculator;
}
namespace Sparrow {
namespace nddo {

namespace UnoCasGeneration {

auto hasOrthogonalBasis(const std::shared_ptr<Core::Calculator>& calc) -> bool;

template<typename FilterType = Utils::SecondQuantization::UnoCI::ThresholdFilter>
auto generate(const std::shared_ptr<Core::Calculator>& calc, FilterType = {},
              const Utils::Settings& steererSettings = OrbitalSteeringSettings{}, const Core::Log& log = Core::Log::silent())
    -> std::tuple<Utils::SecondQuantization::CasSpecifier<Utils::Reference::Restricted>, Eigen::VectorXd>;

} // namespace UnoCasGeneration
} // namespace nddo
} // namespace Sparrow
} // namespace Scine

#endif // SPARROW_UNOCAS_H
